__title__ = 'gksol'
__summary__ = 'Game of Life Tutorial Implementation and Framework'
__url__ = 'https://github.com/MaineKuehn/gksol'

__version__ = '0.2.0'
__author__ = 'Eileen Kuehn, Max Fischer'
__email__ = 'maxfischer2781@gmail.com'
__copyright__ = '2017 %s' % __author__
__keywords__ = 'gridkaschool tutorial game-of-life gol'
